"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.vue3FrameworkPlugin = void 0;
const vfs_1 = require("@previewjs/core/vfs");
const type_analyzer_1 = require("@previewjs/type-analyzer");
const path_1 = __importDefault(require("path"));
const analyze_component_1 = require("./analyze-component");
const vue_reader_1 = require("./vue-reader");
exports.vue3FrameworkPlugin = {
    isCompatible: async (dependencies) => {
        var _a;
        return ((_a = dependencies["vue"]) === null || _a === void 0 ? void 0 : _a.majorVersion) === 3 || !!dependencies["nuxt3"];
    },
    async create() {
        const { default: createVuePlugin } = await Promise.resolve().then(() => __importStar(require("@vitejs/plugin-vue")));
        const { default: vueJsxPlugin } = await Promise.resolve().then(() => __importStar(require("@vitejs/plugin-vue-jsx")));
        const { vueComponentLoaderPlugin } = await Promise.resolve().then(() => __importStar(require("./component-loader-plugin")));
        const { extractVueComponents } = await Promise.resolve().then(() => __importStar(require("./extract-component")));
        const { Parser } = await Promise.resolve().then(() => __importStar(require("acorn")));
        const previewDirPath = path_1.default.resolve(__dirname, "..", "preview");
        return {
            name: "@previewjs/plugin-vue3",
            defaultWrapperPath: "__previewjs__/Wrapper.vue",
            previewDirPath,
            transformReader: (reader, rootDirPath) => (0, vfs_1.createStackedReader)([
                (0, vue_reader_1.createVueTypeScriptReader)(reader),
                (0, vfs_1.createFileSystemReader)({
                    mapping: {
                        from: path_1.default.join(previewDirPath, "modules"),
                        to: path_1.default.join(rootDirPath, "node_modules"),
                    },
                    watch: false,
                }),
            ]),
            componentDetector: (program, filePaths) => {
                const components = [];
                for (const filePath of filePaths) {
                    if (filePath.endsWith(".vue")) {
                        const name = path_1.default.basename(filePath, path_1.default.extname(filePath));
                        components.push({
                            filePath,
                            name,
                            exported: true,
                            offsets: [[0, Infinity]],
                        });
                    }
                    else {
                        components.push(...extractVueComponents(program, filePath));
                    }
                }
                return components;
            },
            componentAnalyzer: ({ typescriptAnalyzer, getTypeAnalyzer }) => (filePath, componentName) => {
                if (filePath.endsWith(".vue")) {
                    return (0, analyze_component_1.analyzeVueComponentFromTemplate)(typescriptAnalyzer, getTypeAnalyzer, filePath);
                }
                else {
                    // TODO: Handle JSX and Storybook stories.
                    return {
                        name: componentName,
                        propsType: type_analyzer_1.UNKNOWN_TYPE,
                        providedArgs: new Set(),
                        types: {},
                    };
                }
            },
            viteConfig: (config) => {
                return {
                    plugins: [
                        vueComponentLoaderPlugin({
                            config,
                        }),
                        createVuePlugin(),
                        vueJsxPlugin(),
                        {
                            name: "previewjs:disable-vue-hmr",
                            async transform(code, id) {
                                if (!id.endsWith(".vue")) {
                                    return null;
                                }
                                // HMR code prevents component loader from receiving
                                // updated preview props, so we disable it.
                                // If you find a better or more reliable way, please
                                // feel free to send a PR :)
                                const matchHmr = /import\.meta\.hot\.accept\((.|\n)*\}\);?/m;
                                return code.replace(matchHmr, "");
                            },
                        },
                        {
                            name: "previewjs:process-define-preview",
                            async transform(code, id) {
                                if (!code.includes("const _sfc_main = ")) {
                                    return;
                                }
                                let parsed;
                                try {
                                    parsed = Parser.parse(code, {
                                        ecmaVersion: "latest",
                                        sourceType: "module",
                                    });
                                }
                                catch (e) {
                                    return null;
                                }
                                // Note: acorn doesn't provide detailed typings.
                                for (const statement of parsed.body || []) {
                                    if (statement.type === "VariableDeclaration") {
                                        for (const declaration of statement.declarations) {
                                            if (declaration.type === "VariableDeclarator" &&
                                                declaration.id.type === "Identifier" &&
                                                declaration.id.name === "_sfc_main" &&
                                                declaration.init.type === "CallExpression" &&
                                                declaration.init.callee.type === "Identifier" &&
                                                declaration.init.callee.name === "_defineComponent" &&
                                                declaration.init.arguments.length > 0 &&
                                                declaration.init.arguments[0].type ===
                                                    "ObjectExpression") {
                                                const properties = declaration.init.arguments[0].properties;
                                                const setupProperty = properties.find((p) => p.key.name === "setup" &&
                                                    p.value.type === "FunctionExpression");
                                                if (!setupProperty) {
                                                    continue;
                                                }
                                                const setupBody = setupProperty.value.body.body;
                                                const definePreviewsCall = setupBody.find((s) => s.type === "ExpressionStatement" &&
                                                    s.expression.type === "CallExpression" &&
                                                    s.expression.callee.type === "Identifier" &&
                                                    s.expression.callee.name === "definePreviews");
                                                if (!definePreviewsCall ||
                                                    definePreviewsCall.expression.arguments.length === 0) {
                                                    continue;
                                                }
                                                const props = definePreviewsCall.expression.arguments[0];
                                                return (code +
                                                    `
                          _sfc_main.previews = ${code.slice(props.start, props.end)};
                          `);
                                            }
                                        }
                                    }
                                }
                                return null;
                            },
                        },
                        {
                            name: "previewjs:optimize-deps",
                            config: () => ({
                                optimizeDeps: {
                                    include: ["vue"],
                                },
                            }),
                        },
                    ],
                    esbuild: {
                        banner: `import { h } from 'vue';`,
                        jsxFactory: "h",
                    },
                    resolve: {
                        alias: {
                            vue: "vue/dist/vue.esm-bundler.js",
                        },
                    },
                };
            },
        };
    },
};
